###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

# -*- coding: utf-8 -*-
from os import path
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager
from Globals import InitializeClass
from OFS.FindSupport import FindSupport
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Settings import ApplicationToolName, ApplicationToolMetaType, DatabaseToolName
from Permissions import RakunView, ManagePortal
from sql import Condition
from Settings import EntityMetaType, InterfaceMetaType, RelationToolName,\
     ApplicationMetaType, MasterDetailMetaType
from RakunBaseAppTool import RakunBaseAppTool

# -------- Rakun Application ---------------------------------------------------
manage_addApplicationForm = PageTemplateFile('zmi/manage_addApplicationForm',
                                             globals())

def manage_addApplication(dispatcher,id,title='',REQUEST=None):
    """Sub Menu add from management screen """
    app = Application(id,title)    
    dispatcher.Destination()._setObject(id,app)
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)
        
class Application(Folder):
    """Store Rakun objects"""
    
    meta_type = ApplicationMetaType

    all_meta_types = ({'name':EntityMetaType,
                       'action':'manage_addProduct/Rakun/manage_addEntityForm',
                       'permission':'View management screens'},
                      #{'name':MasterDetailMetaType,
                      # 'action':'manage_addProduct/Rakun/manage_addMasterDetailFormTemplate',
                      # 'permission':'View management screens'},
                      {'name':InterfaceMetaType,
                       'action':'manage_addProduct/Rakun/manage_addInterfaceForm',
                       'permission':'View management screens'},
                      {'name':ApplicationMetaType,
                       'action':'manage_addProduct/Rakun/manage_addApplicationForm',
                       'permission':'View management screens'}
                      )
    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        
InitializeClass(Application)

class ApplicationTool(Folder, UniqueObject, RakunBaseAppTool):
    """
       Application Tool 
    """
    toolicon = 'zmi/application-tool.gif'
    id = ApplicationToolName
    meta_type = ApplicationToolMetaType
    
    relations = []
    db_versions = {}
    db_migration_script = "migrate_database"
    
    _properties = ({'id':'title', 'type':'string', 'mode':'w'},
                   {'id':'db_migration_script', 'type':'string', 'mode':'w'},)

    manage_options = (Folder.manage_options +\
                      ({'label':'Export',
                        'action':'exportSettings'},                       
                       {'label':'Database Settings',
                        'action':'databaseSettings'},))
    all_meta_types = ({'name':ApplicationMetaType,
                       'action':'manage_addProduct/Rakun/manage_addApplicationForm',
                       'permission':'View management screens'},
                      {'name':'Folder', #backward compatibility
                       'action':'manage_addProduct/OFSP/folderAdd',
                       'permission':'View management screens'}
                      )
    
    security = ClassSecurityInfo()       
    exportSettings = PageTemplateFile('zmi/exportSettings', globals())
    databaseSettings = PageTemplateFile('zmi/databaseSettings', globals())
  
    def index_html(self,REQUEST=None):
        """ """
        return self.rakun_application_view(REQUEST)
    
    security.declareProtected(ManagePortal, 'manage_migrateRelations')
    def manage_migrateRelations(self,REQUEST=None):
        """all relations move to rakun relation tool """
        relation_tool = getToolByName(self, RelationToolName)
        for relation in self.relations:
            relation_tool.addRelation(relation['master_entity'], 
                                      relation['master_fields'].split(','),
                                      relation['detail_entity'],
                                      relation['foreign_field'],
                                      relation['relation'])
        if REQUEST:
            message = 'All relations are migrated.'
            return self.relationSettings(self,REQUEST,
                                         manage_tabs_message=message)
            
            
            
    security.declareProtected(RakunView, 'relation_search')
    def relation_search(self,master_entity=None,detail_entity=None,
                        foreign_field=None):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.relation_search(master_entity,detail_entity,
                                             foreign_field)
    
    security.declareProtected(RakunView,'getRelation')
    def getRelation(self,foreign):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.getRelation(foreign)
            
    security.declareProtected(RakunView,'getMasterFields')
    def getMasterFields(self, foreign):
        """ """
        relation_tool = getToolByName(self, RelationToolName)
        return relation_tool.getMasterFields(foreign)
    
    security.declareProtected(RakunView,'getMasterEntity')    
    def getMasterEntity(self, foreign):
        """ """
        relation = self.getRelation(foreign)
        if relation:
            master_entity = self.restrictedTraverse(\
                relation[0]['master_entity'])
            return master_entity
        
    security.declareProtected(RakunView,'getEntityObject')    
    def getEntityObject(self, entity_path):
        return self.unrestrictedTraverse(entity_path)
    
    security.declareProtected(RakunView,'getJoinCondition')
    def getJoinCondition(self,foreign):
        """ """
        master_entity = self.getMasterEntity(foreign)
        if master_entity:
            primary_key = master_entity.getPrimaryField()
            return Condition(primary_key,'=',foreign)
        
    security.declareProtected(RakunView,'getAllEntities')
    def getAllEntities(self):
        return FindSupport.ZopeFind(FindSupport(self), self, [],
                                    [EntityMetaType],search_sub=1)
    
    security.declareProtected(RakunView,'getAllDetailEntities')
    def getAllDetailEntities(self,master_entity):
        """ """
        relations = self.relation_search(master_entity=master_entity)
        return map(lambda r: r['detail_entity'], relations)
    
    security.declareProtected(RakunView,'getAllInterface')
    def getAllInterface(self,master_entities=[]):
        """ """
        interfaces = FindSupport.ZopeFind(FindSupport(self), self, [],
                                          [InterfaceMetaType],search_sub=1)
        if master_entities:
            return filter(lambda interface:interface[1].master_entity in\
                          master_entities, interfaces)
        else:
            return interfaces
    
    security.declareProtected(ManagePortal, 'manage_export')
    def manage_export(self,ids=[],export_path="",REQUEST=None):
        """rakun applications export method"""
        message = StringIO()
        if not ids:
            if REQUEST: 
                message.write("Select at least one module")
                return self.exportSettings(self,REQUEST,
                                           manage_tabs_message=message.getvalue())
        if not export_path:
            if REQUEST: 
                message.write("Enter an export path")
                return self.exportSettings(self,REQUEST,
                                           manage_tabs_message=message.getvalue())
        
        message.write("These modules are exported to %s\n"%export_path)
        #for id in self.objectIds():             
        for id in ids:
            ob=self._getOb(id)
            if ob.meta_type != 'Filesystem Directory View':
                f = path.join(export_path, '%s.%s' % (id, 'zexp'))
                ob._p_jar.exportFile(ob._p_oid, f)
                message.write("%s \n"%(f))
            
        if REQUEST:            
            return self.exportSettings(self,REQUEST,
                                       manage_tabs_message=message.getvalue())
        
     
    
    #---------------------------------------------------------------------------
    
    security.declareProtected(ManagePortal, 'manage_getDatabaseInfo')
    def manage_getDatabaseInfo(self):
        """ """
        db_info = []
        connectionids = []
        for path,entity in self.getAllEntities():
            connectionid = entity.getConnectionId()
            if not connectionid in connectionids:
                dbobj = entity.getDBObject()
                db_info.append({'dbfamily' : entity.getDBFamily(),
                                'connectionid' : connectionid,
                                'version_no' : dbobj.getVersion()})
                connectionids.append(connectionid)
        return db_info
    
    security.declareProtected(ManagePortal, 'manage_setDatabaseVersion')
    def manage_setDatabaseVersion(self,databases=[],REQUEST=None):
        """ """
        self.db_versions = {}
        for database in databases:
            self.db_versions[database['connectionid']] = database['version_no']
        self.db_versions = self.db_versions
        if REQUEST:
            message="Application Tool Database Versions Updated"
            return self.databaseSettings(self,REQUEST,
                                         manage_tabs_message=message)  
    
    security.declareProtected(ManagePortal, 'manage_migrate')
    def manage_migrate(self):
        """ """
        dbtool = getToolByName(self, DatabaseToolName)
        dbobj = dbtool.getDBObject(dbfamily,connectionid)
        dbobj.setVersion(version_no)
  
    
    security.declareProtected(ManagePortal, 'sync_sequences')
    def sync_sequences(self,REQUEST=None):
        """ sycronize database sequences
        """
        for entityid,entity in self.getAllEntities():
            entity.sync_sequence()
        if REQUEST:
            message="All sequences are syncronized."
            return self.databaseSettings(self,REQUEST,
                                         manage_tabs_message=message) 
        
InitializeClass(ApplicationTool)    


