###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

# -*- coding: utf-8 -*-
import sys
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Permissions import RakunInsert, RakunDelete, RakunUpdate,\
     RakunAccessDatabase, RakunView, ManagePortal
from Errors import RakunInsertError, RakunDeleteError, RakunUpdateError,\
     RakunValidationError
from Products.Formulator.Errors import FormValidationError, ValidationError
from Settings import DatabaseToolName, ApplicationToolName
from sql.Functions import Function
from sql import Max,As
from RequestManagement import RequestManager

REQUEST_ARGS = '_arguments'
REQUEST_FUNC_ARGS = '_functions'

class RPCResponse:
    
    def __init__(self):
        self.error = ''
        self.error_type = ''
        self.result = '' 
        self.message = ''

    def setError(self,exception,entity=None):
        if entity and isinstance(exception.value,basestring):
            self.error = entity.get_i18n_message(exception.value)
        else:
            self.error = exception.value
        self.error_type = exception.error_type
                
    def __call__(self):
        return {'error':self.error,
                'error_type':self.error_type,
                'result':self.result,
                'message':self.message
                }
    


class DataManager(RequestManager):
    
    security = ClassSecurityInfo()
    
    def _setInsertContext(self,**args):
        self._setContext(**args)
        request = self.get_arguments()
        dbobject = self.getDBObject()
        for field in self.get_fields():
            if field.get_value('insert'):
                if field.has_value("isautoincrement") and \
                   field.get_value("isautoincrement"):
                    value = str(dbobject.getAutoIncrementValue(self.getEntityName(),field.getName()))
                else:
                    value = field._get_default(field.getId(),None,request)
                args[field.getId()] = value
        self._setContext(**args)
        
    def _setUpdateContext(self,**args):
        self._setContext(**args)
        request = self.get_arguments()
        for field in self.get_fields():
            if args.has_key(field.getId()):
                if field.get_value('update'):
                    value = field._get_default(field.getId(),None,request)
                    args[field.getId()] = value
        self._setContext(**args)
    
    # Public Methods    
    security.declareProtected(RakunAccessDatabase, 'getDBObject')
    def getDBObject(self):
        dbtool = getToolByName(self, DatabaseToolName)
        return dbtool.getDBObject(self.getDBFamily(),self.getConnectionId())
    
    def getRowNumber(self,primary_value,fields=[],condition=None,orderby=[],
                     grpupby=[]):
        """ """
        dbobject = self.getDBObject()
        if not fields:
            fields = self.get_fields()
        return dbobject.getRowNumber(self, primary_value, fields, condition,
                                     orderby, grpupby)
    
    security.declareProtected(RakunInsert, 'insert')
    def insert(self, **args):
        self.validateInsert(**args) # _setInsertContext validation içinde çağrılıyor
        try:                       
            self._execMethod('before_insert')
            insert_values = []
            request = self.get_arguments()
            dbobject = self.getDBObject()
            for field in self.get_fields():
                if field.get_value('insert'):
                    if field.getId() in request.keys():
                        value = request.get(field.getId())
                        insert_values.append((field,value))
            dbobject.insert(self.getEntityName(),insert_values)            
            pkey_value = request.get(self.getPrimaryKey())
            self._execMethod('on_insert')
            self.set_message( self.get_i18n_message(self.insert_message) )
            return pkey_value
        except:
            self._logException()
            raise RakunInsertError(str(sys.exc_type))        
        
    
    security.declareProtected(RakunUpdate, 'update')
    def update(self, condition, **args):
        self.validateUpdate(condition, **args)
        try:  
            self._execMethod('before_update')
            request = self.get_arguments()
            condition = self.get_condition()
            update_values = []
            dbobject = self.getDBObject()
            for field in self.get_fields():
                if field.get_value('update'):
                    if field.getId() in request.keys():
                        value = request.get(field.getId())
                        update_values.append( (field,value) )
            if update_values:
                dbobject.update(self.getEntityName(), update_values, condition)                                   
            self._execMethod('on_update')
            self.set_message( self.get_i18n_message(self.update_message) )
        except:
            self._logException()
            raise RakunUpdateError(str(sys.exc_type))
            
    security.declareProtected(RakunDelete, 'delete')
    def delete(self, condition):
        self.validateDelete(condition)
        try:
            self._execMethod('before_delete')
            request = self.get_arguments()
            condition = self.get_condition()
            dbobject = self.getDBObject()
            dbobject.delete(self.getEntityName(),condition)
            self._execMethod('on_delete')
            self.set_message( self.get_i18n_message(self.delete_message) )
        except:
            self._logException()
            raise RakunDeleteError(str(sys.exc_type))
        
    security.declareProtected(RakunView, 'select')
    def select(self, fields=[], condition=None, orderby=[], groupby=[], 
               offset=0, limit=0):
        """ """     
        
        dbobject = self.getDBObject()                     
       
        if not fields:    
            fields = self.get_fields()
            
            
        return dbobject.select(self, fields, condition, orderby, groupby, 
                               offset,limit)
        
    security.declareProtected(RakunView, 'rowCount')
    def rowCount(self, condition=None, groupby=[]):
        """ """    
        dbobject = self.getDBObject()
        return dbobject.rowCount(self, condition, groupby)               
        
    security.declareProtected(RakunInsert, 'validateInsert')
    def validateInsert(self, **args):
        """   """
        self._setInsertContext(**args)
        request = self.get_arguments()
        try:
            self.validate_all_to_request(request)
        except FormValidationError,exception:
            errors = {}
            for error in exception.errors:
                errors[error.field.getId()] = str(error.error_text)
            raise RakunValidationError(errors)        
        self._execMethod('validate_insert')        
    
    security.declareProtected(RakunUpdate, 'validateUpdate')
    def validateUpdate(self, condition, **args):
        """ """
        self._setUpdateContext(**args)
        exception_errors = []
        request = self.get_arguments()
        self.set_condition(condition)
        for field in self.get_fields():
            if field.getId() in request.keys():
                try:
                    value = request[field.getId()]
                    if not isinstance(value,Function):
                        field.validate(request)
                except ValidationError, e:
                    exception_errors.append(e) 
                except:
                    self._logException()
        if exception_errors:
            errors = {}
            for error in exception_errors:
                errors[error.field.getId()] = str(error.error_text)
            raise RakunValidationError(errors)
        try:
            self._execMethod('validate_update')    
        except (RakunUpdateError,RakunValidationError), e:
            raise e
        
    
    security.declareProtected(RakunDelete, 'validateDelete')
    def validateDelete(self, condition):   
        self.set_condition(condition)
        try:
            self._execMethod('validate_delete') 
        except (RakunDeleteError,RakunValidationError), e:
            raise e
    
    
                        
    # RPC Calls
    security.declareProtected(RakunInsert, 'rpcValidateInsert')
    def rpcValidateInsert(self,args):
        """
        """        
        response = RPCResponse()
        try:
            response.result = self.validateInsert(**args)
        except (RakunInsertError,RakunValidationError), e:
            response.setError(e,self) 
            self._logException()
        return response()
    
    security.declareProtected(RakunInsert, 'rpcInsert')
    def rpcInsert(self, args):
        """
        """
        response = RPCResponse()
        try:
            response.result = self.insert(**args)
            response.message = self.get_message()
        except (RakunInsertError,RakunValidationError),e:
            response.setError(e,self)
            response.result = 0
            self._logException()
        return response()
    
    security.declareProtected(RakunDelete, 'rpcValidateUpdate')
    def rpcValidateUpdate(self, condition, args):
        """ """        
        response = RPCResponse()
        try:
            response.result = self.validateUpdate(condition,**args)
        except (RakunUpdateError,RakunValidationError), e:
            response.setError(e,self)
            response.result = 0
            self._logException()
        return response()
    
    security.declareProtected(RakunInsert, 'rpcUpdate')
    def rpcUpdate(self, condition, args):
        """
        """        
        response = RPCResponse()
        try:            
            self.update(condition, **args)
            response.message = self.get_message()
            response.result = 1
        except (RakunUpdateError,RakunValidationError),e:
            response.setError(e,self) 
            response.result = 0
            self._logException()
        return response()    
    
    security.declareProtected(RakunDelete, 'rpcValidateDelete')
    def rpcValidateDelete(self,condition):
        """ 
        """        
        response = RPCResponse()
        try:
            response.result = self.validateDelete(condition)
        except (RakunDeleteError,RakunValidationError), e:
            response.setError(e,self)
            response.result = 0
            self._logException()
        return response()
      
    security.declareProtected(RakunDelete, 'rpcDelete')
    def rpcDelete(self,condition):
        """
        """
        response = RPCResponse()
        try:
            response.result = self.delete(condition)
            response.message = self.get_message()
            response.result = 1
        except (RakunDeleteError,RakunValidationError), e:
            response.setError(e,self)   
            response.result = 0
            self._logException()
        return response()
        
    security.declareProtected(RakunView, 'rpcSelect')
    def rpcSelect(self, fields=[], condition=None, orderby=[], groupby=[], 
                  offset=0,limit=0):
        """ """
        response = RPCResponse()
        try:
            response.result = self.select(fields, condition, orderby, groupby, 
                                          offset, limit)
        except :
            response.result = []
            response.error = 'RakunError: rpcSelect'
            self._logException()
        return response()
    
    security.declareProtected(RakunView, 'rpcRowCount')
    def rpcRowCount(self, condition=None, groupby=[]):
        """ """
        response = RPCResponse()        
        response.result = self.rowCount(condition,groupby)
        return response()
    
    # Veritabanı import/export'unda sequence doğru iletilemediği için
    security.declareProtected(ManagePortal, 'sync_sequence')
    def sync_sequence(self):
        pfield = self.getPrimaryField()
        rec = self.select([As(Max(pfield),'max_pvalue')])
        if rec:
            max_pvalue = rec[0]['max_pvalue']
            dbobj = self.getDBObject()
            tableid = self.getEntityName()
            fieldid = self.getPrimaryKey()
            try: currval = dbobj.getLastIncrementValue(tableid,fieldid)
            except: currval = dbobj.getAutoIncrementValue(tableid,fieldid)
            if not currval:
                currval = dbobj.getAutoIncrementValue(tableid,fieldid)
            while currval < max_pvalue:
                currval = dbobj.getAutoIncrementValue(tableid,fieldid)
        return 1
  
InitializeClass(DataManager) 