###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

from Products.Formulator.DummyField import fields

class GRIDProperties:
    """Field GRID Properties
    """
    
    property_names = ['issearchable','grid_hidden','summary','grid_css_class']
    
    issearchable = fields.CheckBoxField('issearchable',
                                        title='Searchable',
                                        default=False,
                                        required=0)
    
    grid_hidden = fields.CheckBoxField('grid_hidden',
                                       title='Hidden',
                                       default=False,
                                       required=0)
    
    summary = fields.CheckBoxField('summary',
                                   title='Summary',
                                   default=False,
                                   required=0)

    grid_css_class = fields.StringField('grid_css_class',
                                        title='CSS Class',
                                        required=0)

FieldGRIDPropertiesInstance = GRIDProperties()

class FORMProperties:
    """Field FORM Properties
    """
    
    property_names = ['helperapp','relationcond']
    
    helperapp = fields.StringField('helperapp',
                                   title='Helper Application',
                                   required=0)
    
    relationcond = fields.StringField('relationcond',
                                      title='Relation Condition',
                                      required=0)

FieldFORMPropertiesInstance = FORMProperties()


class DBProperties:
    """Field Database Properties
    """

    property_names = ['name', 'type', 'isautoincrement', 'insert', 'update']

    name = fields.StringField('name',
                              title='Column Name',
                              required=0)

    type = fields.ListField('type',
                            title='Type',
                            items=[('String', 'string'),
                                   ('Text','text'),
                                   ('Boolean', 'boolean'),
                                   ('Integer', 'integer'),
                                   ('Float','float'),
                                   ('Date','date'),
                                   ('Money','money'),
                                   ('File','file')],
                            required=1,
                            size=1,
                            default='string')
    
           
    isautoincrement = fields.CheckBoxField('isautoincrement',
                                    title='Is Auto Increment',
                                    default=0,
                                    required=0)
    
    insert = fields.CheckBoxField('insert',
                                  title='Insert Allowed',
                                  default=1,
                                  required=0)
    
    update = fields.CheckBoxField('update',
                                  title='Update Allowed',
                                  default=1,
                                  required=0)
    
FieldDBPropertiesInstance = DBProperties() 


class FileDBProperties(DBProperties):
    
    property_names = ['name','filename_column','type','insert','update']
    
    name = fields.StringField('name',
                              title='Blob Column',
                              required=1)    
    
    filename_column = fields.StringField('filename_column',
                                         title='Filename Column',
                                         required=1)
    
FileFieldDBPropertiesInstance = FileDBProperties()


class ImageFORMProperties(FORMProperties):

    property_names = ['helperapp','relationcond','width','height']
    
    width = fields.IntegerField('width',
                                title='Width',
                                required=0)  
    
    height = fields.IntegerField('height',
                                 title='Height',
                                 required=0)  

ImageFieldFormPropertiesInstance = ImageFORMProperties()    