###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Image import cookId
from Products.Formulator import StandardFields
from Products.Formulator.Field import ZMIField
from Products.Formulator import Widget, Validator
from Products.Formulator.StandardFields import create_datetime_text_sub_form
from Permissions import RakunAccessDatabase,RakunView
from DateTime import DateTime
from Errors import RakunValidationError
from RakunUtils import translate
from Settings import i18ndomain
from IObjects import ISql, IRpc
from Condition import Eq
import FieldProperties
import Widgets

def validate(self, field, key, REQUEST):
  value = REQUEST.get(key, "")
  if not field.get_value('whitespace_preserve'):
      try: value = value.strip()
      except: pass
  if field.get_value('required') and value == "":
      self.raise_error('required_not_found', field)
  return value

Validator.StringBaseValidator.validate = validate

default_types = {"RakunCheckBoxField":"boolean",
                 "RakunIntegerField":"integer",
                 "RakunRangedIntegerField":"integer",
                 "RakunFloatField":"float",
                 "RakunDateTimeField":"date",
                 "RakunFileField":"file",
                 "RakunImageField":"file",
                 }

class BaseField(ZMIField):
  """"""
  __implements__ = (ISql,IRpc,)
  
  dbproperties = FieldProperties.FieldDBPropertiesInstance
  gridproperties = FieldProperties.FieldGRIDPropertiesInstance
  formproperties = FieldProperties.FieldFORMPropertiesInstance
      
  security = ClassSecurityInfo()   
  
  security.declareProtected(RakunView,"title_or_id")
  def title_or_id(self):
    """ """
    title = ZMIField.title_or_id(self)
    return str(title)
  
  def manage_afterAdd(self, item, container):
    type = default_types.get(item.meta_type, None)
    if type:
      item._edit({'type':type})
      
    return ZMIField.manage_afterAdd(self, item, container)
  
  #--------IRpc Implementation -------------------------------------------------
  def getProperties(self):
    properties = {}
    properties['id'] = properties['inputname'] = self.getId()
    properties['title'] = self.get_value('title')
    properties['type'] = self.get_value("type")
    properties['hidden'] = self.get_value('hidden')
    properties['grid_hidden'] = self.get_value('grid_hidden')
    properties['grid_css_class'] = self.get_value('grid_css_class')
    properties['summary'] = self.get_value('summary')
    properties['__class__'] = 'Field'
    properties['path'] = '/'.join(self.getPhysicalPath())
    return properties
  
  def dump(self, marshaller, write):
        properties = self.getProperties()        
        marshaller.dump_struct(properties, write)
        
  def getObject(self):
        """ """
        return self
      
  # -------ISql Implementation--------------------------------------------------
  def render_sql_value(self, dbobject, value):
    if ISql.isImplementedBy(value):
      return value.get_sql_field_name()
    else:
      value = dbobject.setEscapeChars(value)
      renderer = getattr(dbobject, '%s_render'%self.get_value('type'),dbobject.dummy_render)
      return renderer(value)
  
  def render_sql_insert_field(self):
    return self.getName()
     
  def render_sql_update_field(self):
    return self.getName()
    
  def render_sql_field(self):
    return '%s.%s' %(self.getEntityName(), self.getName())

  def render_sql_function(self):
    return self.render_sql_field()
    
  def get_sql_field_name(self):
    return self.getName()

  def get_db_type(self):
    return self.get_value('type')
  
  def get_sql_field_alias(self):
    return self.getId()
    
  def render_sql_table(self):
    return self.getEntityName()

  def get_sql_join_condition(self):
    return None
    
  def render_sql_condition(self):
    return self.render_sql_field()
  
  def render_sql_groupby(self):
    return self.render_sql_field()
  
  def render_sql_orderby(self):
    return self.render_sql_field()
  
  # ---------------------------------------------------------------------------#
  
  security.declareProtected(RakunView, 'get_value')
  def get_value(self, id, **kw):
    value = ZMIField.get_value(self, id, **kw)
    if id in ['title', 'description']:
      #if hasattr(value,'translate'):
      #  return value.translate().encode(self.get_form_encoding())
      if hasattr(value,'default'):
        value = value.default
      value = self.get_i18n_message(value)
    return value
      
  
  security.declareProtected(RakunAccessDatabase,"getName")
  def getName(self):
    """ """
    name = self.get_value('name')
    return name or self.getId()

  security.declareProtected(RakunAccessDatabase,"getEntity")  
  def getEntity(self):
    """ """
    return self.aq_inner.aq_parent
    
  security.declareProtected(RakunView, 'get_error_message')
  def get_error_message(self, name, want_message_id=True):
    error_message = ZMIField.get_error_message(self,name, want_message_id)
    try: 
      default_message = error_message.default.encode("utf-8")
    except:
      default_message = error_message.encode("utf-8")
      
    try: 
      i18n_message = error_message.translate().encode("utf-8")
    except:
      i18n_message = error_message.encode("utf-8")
      
    if i18n_message.startswith('Unknown error:'):
      i18n_message = name
    if i18n_message == default_message:
      return translate(i18ndomain,default_message,
                       context=self,default=default_message,
                       as_unicode=1).encode("utf-8")
    else:
      return i18n_message
    
  security.declareProtected(RakunView, 'generate_field_key')
  def generate_field_key(self, validation=0):
    """Generate the key Silva uses to render the field in the form.
    """
    if self.field_record is None:
      return self.id
    elif validation:
      return self.id
    elif isinstance(self.widget, MultiItemsWidget):
      return "%s.%s:record:list" % (self.field_record, self.id)
    else:
      return '%s.%s:record' % (self.field_record, self.id)

  def relationCondition(self, REQUEST=None):
    """run relation_condition script """
    if REQUEST is not None:
      entity = self.getEntity()
      entity._setContext(**REQUEST)
    condition = self.get_value('relationcond')
    if isinstance(condition,basestring):
      return None
    else:
      return condition
  
  def rpcRelationCondition(self, REQUEST=None):
    """relationCondition rpc interface"""
    condition = self.relationCondition(REQUEST)
    if condition:
      return condition.toRpc()
    else:
      return ''

    
    
class StringField(BaseField, StandardFields.StringField):
  """ """
  meta_type = "RakunStringField"  
  

class PasswordField(BaseField, StandardFields.PasswordField):
  """ """
  meta_type = "RakunPasswordField"

  
class EmailField(BaseField, StandardFields.EmailField):
  """ """
  meta_type = "RakunEmailField"

  
class PatternField(BaseField, StandardFields.PatternField):
  """ """
  meta_type = "RakunPatternField"

  
class CheckBoxField(BaseField, StandardFields.CheckBoxField):
  """ """
  meta_type = "RakunCheckBoxField"
  
class IntegerField(BaseField, StandardFields.IntegerField):
  """ """
  meta_type = "RakunIntegerField"
  
class RangedIntegerField(BaseField,StandardFields.RangedIntegerField):
  """ """
  meta_type = "RakunRangedIntegerField"
  
class FloatField(BaseField,StandardFields.FloatField):
  """ """
  meta_type = "RakunFloatField"
  
class TextAreaField(BaseField,StandardFields.TextAreaField):
  """ """
  meta_type = "RakunTextAreaField"

  
class RawTextAreaField(BaseField,StandardFields.RawTextAreaField):
  """ """
  meta_type = "RakunRawTextAreaField"

  
class ListField(BaseField,StandardFields.ListField):
  """ """
  meta_type = "RakunListField"

  
class MultiListField(BaseField,StandardFields.MultiListField):
  """ """
  meta_type = "RakunMultiListField"

  
class LinesField(BaseField,StandardFields.LinesField):
  """ """
  meta_type = "RakunLinesField"

  
class RadioField(BaseField,StandardFields.RadioField):
  """ """
  meta_type = "RakunRadioField"

  
class MultiCheckBoxField(BaseField,StandardFields.MultiCheckBoxField):
  """ """
  meta_type = "RakunMultiCheckBoxField"

  
class FileField(BaseField,StandardFields.FileField):
  """ """
  meta_type = "RakunFileField"
  widget = Widgets.FileWidgetInstance
  
  dbproperties = FieldProperties.FileFieldDBPropertiesInstance
  
  def render_sql_insert_field(self):    
    return '%s,%s' %(self.get_value('name'),self.get_value('filename_column'))
  
  def render_sql_value(self, dbobject, file):
    try:
      title_or_id = file.title_or_id()
      data = str(file.data)
    except:
      title_or_id = ""
      data = BaseField.render_sql_value(self, dbobject,None)
    name = BaseField.render_sql_value(self, dbobject, title_or_id) 
    blob = dbobject.escapeBlob(data)
    return """%s,'%s'""" %(blob, name)
  
  def render_sql_field(self):
    return '%s.%s' %(self.getEntityName(), self.get_value('filename_column'))
  
  def get_sql_field_name(self):
    return self.get_value('filename_column')
  
  def getBlobObject(self, primary_value, REQUEST=None,RESPONSE=None):
    """ """     
    from StringIO import StringIO
    from OFS.Image import File
    dbobject = self.getDBObject()
    primary_field = self.getPrimaryField()
    condition = "%s=%s" %(primary_field.getName(), primary_value)
    query = "SELECT %s FROM %s WHERE %s" \
          %(self.render_sql_insert_field(),self.getEntityName(),condition)
    result = dbobject.execQuery(query)
    if result:
      filename = result[0][self.get_value('filename_column')]
      data = result[0][self.get_value('name')]
      f = StringIO(dbobject.unescapeBlob(data))
      file = File(filename,filename,f)
      if REQUEST is not None: 
        RESPONSE.setHeader('Content-disposition','inline;filename=%s'%filename)
        return file.index_html(REQUEST,RESPONSE)
      else:
        return file
      
  #security.declareProtected(RakunView, 'fileUpload')
  def fileUpload(self,REQUEST=None):
    """ """
    file = REQUEST.get("file",None)
    if file:
      date = DateTime()
      id = "%s%s%s%s%s%s" %(date.year(),date.month(),date.day(),
                            date.hour(),date.minute(),date.second())
      id = id.replace(".","")
      filename,dummy = cookId('','',file)
      self.temp_folder.manage_addFile(id,title=filename,file=file)
      zfileobj = getattr(self.temp_folder,id)
      REQUEST.set('fileurl',"/".join(zfileobj.getPhysicalPath()))
      REQUEST.set('filename',filename)
      REQUEST.set('success',"1")
    else:
      REQUEST.set('success',"0")
      
    return self.filehelper_template(REQUEST)
    

class ImageField(FileField):
  """ """
  meta_type = "RakunImageField"
  widget = Widgets.ImageWidgetInstance 
  
  formproperties = FieldProperties.ImageFieldFormPropertiesInstance
  
  
class LinkField(BaseField,StandardFields.LinkField):
  """ """
  meta_type = "RakunLinkField"

  
class LabelField(BaseField,StandardFields.LabelField):
  """ """
  meta_type = "RakunLabelField"


def datetime_validate(field, key, REQUEST):
  value = REQUEST.get(key)
  if value:
    try: 
      value = DateTime(value)
    except: 
       Validator.DateTimeValidatorInstance.raise_error('not_datetime', field)
    fieldid = field.getId()
    # Get values and keys
    year = str(value.year()).zfill(4)
    year_key = field.generate_subfield_key('year', validation=1)
    month = str(value.month()).zfill(2)
    month_key = field.generate_subfield_key('month', validation=1)
    day = str(value.day()).zfill(2)
    day_key = field.generate_subfield_key('day', validation=1)
    hour = str(value.hour()).zfill(2)
    hour_key = field.generate_subfield_key('hour', validation=1)
    minute = str(value.minute()).zfill(2)
    minute_key = field.generate_subfield_key('minute', validation=1)
    # Set request
    REQUEST.set(year_key, year )
    REQUEST.set(month_key, month )
    REQUEST.set(day_key, day )
    REQUEST.set(hour_key, hour )
    REQUEST.set(minute_key, minute )
  field.checkSubform()
  return Validator.DateTimeValidatorInstance.validate(field, key, REQUEST)
  
DateTimeValidatorInstance = Validator.DateTimeValidator()
DateTimeValidatorInstance.validate = datetime_validate

def datetime_render(field, key, value, REQUEST):  
  field.checkSubform()
  return field.render_calendar_widget(formname='',inputname=field.getId(),inputvalue=value)
  
  
DateTimeWidgetInstance = Widget.DateTimeWidget()
DateTimeWidgetInstance.render = datetime_render

class DateTimeField(BaseField,StandardFields.DateTimeField):
  """ """
  meta_type = "RakunDateTimeField"
  widget = DateTimeWidgetInstance
  validator = DateTimeValidatorInstance

  def checkSubform(self):
    if not hasattr(self, 'sub_form'):
      self.sub_form = create_datetime_text_sub_form()
  
  
  

  
  

