###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

# -*- coding: utf-8 -*-
from os import path
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from Globals import InitializeClass
from OFS.Cache import Cacheable
from OFS.OrderedFolder import OrderedFolder
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.GlobalTranslationService import\
     getGlobalTranslationService
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.utils import UniqueObject
from Settings import MenuToolName, MenuToolMetaType, MenuItemMetaType,\
     SubMenuMetaType, MenuSeparatorMetaType, i18ndomain
from Permissions import RakunView, ManagePortal
from RakunUtils import translate
from RakunBaseAppTool import RakunBaseAppTool

#def translateX(domain, msgid, context, target_language):    
#    ts = getGlobalTranslationService()    
#    return ts.utranslate(domain, msgid, context=context, #default=msgid,target_language=target_language)

#--------- Menu Separator ------------------------------------------------------
manage_addMenuSeparatorForm = PageTemplateFile('zmi/manage_addMenuSeparatorForm',
                                               globals())
def manage_addMenuSeparator(dispatcher,id,REQUEST=None):
    """Menu Item add from management screen """
    separator = MenuSeparator(id)
    dispatcher.Destination()._setObject(id,separator)
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)        
        
class MenuSeparator(SimpleItem):
    """ """
    
    meta_type = MenuSeparatorMetaType
    
    def __init__(self, id):
        self.id = id
        
    # render methods
    def render_JSCookMenu(self):
        """ """
        return "_cmSplit"
InitializeClass(MenuSeparator)    


# -------- Menu Item -----------------------------------------------------------
manage_addMenuItemForm = PageTemplateFile('zmi/manage_addMenuItemForm',
                                          globals())

def manage_addMenuItem(dispatcher,id,title='',REQUEST=None):
    """Menu Item add from management screen """
    menu = MenuItem(id,title)    
    dispatcher.Destination()._setObject(id,menu)
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)

class MenuItem(SimpleItem, PropertyManager):
    """ """
    meta_type = MenuItemMetaType
    
    manage_options = (PropertyManager.manage_options +\
                      SimpleItem.manage_options)
    
    _properties = ({'id':'title', 'type': 'string','mode':'w'},
                   {'id':'description', 'type': 'string','mode':'w'},
                   {'id':'menu_icon', 'type': 'string','mode':'w',
                    'label':'Menu icon'},
                   {'id':'hover_icon', 'type': 'string','mode':'w',
                    'label':'Hover Menu icon'},
                   {'id':'parameter', 'type': 'string','mode':'w',
                    'label':'Parameter(for URL)'},
                   {'id':'path', 'type': 'string','mode':'w',
                    'label':'Relative Object Path'},
                   {'id':'target', 'type': 'selection','mode':'w',
                    'select_variable':'target_option','label':'Target'},
                   )
    target_option = ['_parent','_blank']
        
    def __init__(self,id,title=''):
        self.id = id
        self.title = title
        self.description = ''        
        self.menu_icon = ''
        self.hover_icon = ''
        self.parameter = ''        
        self.path = ''
        self.target = '_parent'    
   
    # render methods
    def render_JSCookMenu(self):
        """render JSCookMenu suported data structure""" 
        obj = None
        if self.path:
            try:
                obj = self.unrestrictedTraverse(self.path)
            except:
                pass
        # security check !!!
        permission = True
        if self.security_check and obj:
            security = getSecurityManager()
            permission = security.checkPermission(RakunView, obj)
        # render jsmenu    
        if obj and permission:
            url = obj.absolute_url()
            if self.parameter:
                url += '?%s' %self.parameter
            title = self.title or obj.title_or_id()
            description = self.description or obj.title_or_id()
            if self.menu_icon and self.hover_icon:
                icon = """<img class="seq1" src="%s"/>
                          <img class="seq2" src="%s"/>"""\
                     %(self.menu_icon, self.hover_icon)
            elif self.menu_icon:
                icon = '<img src="%s" />' %self.menu_icon
            else:
                icon = '&nbsp;'
            # i18n support !!! 
            i18n_domain = self.i18n_domain or i18ndomain            
            i18n_title = translate(i18n_domain,msgid=title,context=self, 
                                   as_unicode=1)
            i18n_description = translate(i18n_domain, msgid=description,
                                         context=self, as_unicode=1)
            
            return "['%s','%s','%s','%s','%s']" %(icon,i18n_title, url,\
                                                  self.target, i18n_description)
        else:
            return ""        
InitializeClass(MenuItem)

# --------Sub Menu -------------------------------------------------------------
manage_addSubMenuForm = PageTemplateFile('zmi/manage_addSubMenuForm', globals())

def manage_addSubMenu(dispatcher,id,title='',REQUEST=None):
    """Sub Menu add from management screen """
    menu = SubMenu(id)
    menu.initialize(title)
    dispatcher.Destination()._setObject(id,menu)
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)
        
class SubMenu(OrderedFolder):
    
    meta_type = SubMenuMetaType
    
    all_meta_types = ({'name':SubMenuMetaType,
                       'action':'manage_addProduct/Rakun/manage_addSubMenuForm',
                       'permission':'View management screens'},
                      {'name':MenuItemMetaType,
                       'action':'manage_addProduct/Rakun/manage_addMenuItemForm',
                       'permission':'View management screens'},
                      {'name':MenuSeparatorMetaType,
                       'action':'manage_addProduct/Rakun/manage_addMenuSeparatorForm',
                       'permission':'View management screens'})

    _properties = ({'id':'title', 'type': 'string','mode':'w'},
                   {'id':'description', 'type': 'string','mode':'w'},
                   {'id':'menu_icon', 'type': 'string','mode':'w',
                    'label':'Menu icon'},                   
                   {'id':'i18n_domain', 'type': 'string','mode':'w',
                    'label':'I18N Domain'},
                   )

    def initialize(self,title):
        self.title = title
        self.description = ''
        self.menu_icon = ''        
        self.in18n_domain = i18ndomain

    # render methods
    def render_JSCookMenu(self):
        """render JSCookMenu suported data structure"""  
        i18n_domain = self.i18n_domain or i18ndomain
        childs = map(lambda child: child.render_JSCookMenu(),
                     self.objectValues())
        childs = filter(lambda child:child,childs)        
        if childs:
            def isEmptyMenu(result,item):
                if result and (item=='_cmSplit'): return True
                else: return False
                
            if childs[0] != '_cmSplit':
                pass
            elif reduce(isEmptyMenu, childs):
                return ""
            
            childs = ','.join(childs)
            if self.id == MenuToolName: # top of menus                
                JSCookMenu = "[%s]" %childs
                return JSCookMenu.encode('utf8')
            else:
                title = self.title_or_id()
                description = self.description or self.title_or_id()               
                if self.menu_icon:
                    icon = '<img src="%s" />' %self.menu_icon
                else:
                    icon = '&nbsp;' 
                # i18n support !!!                               
                i18n_title = translate(i18n_domain, msgid=title, context=self, 
                                       as_unicode=1)
                i18n_description = translate(i18n_domain,msgid=description,
                                             context=self, as_unicode=1)

                return "['%s','%s','','','%s',%s]"\
                       %(icon,i18n_title,i18n_description,childs)          
InitializeClass(SubMenu)

# -------- Rakun Menu Tool -----------------------------------------------------
class MenuTool(UniqueObject, SubMenu, RakunBaseAppTool):
    """ """
    id = MenuToolName
    meta_type = MenuToolMetaType
    toolicon = 'zmi/menu_item.gif'
    menu_icon = ''
    security_check = True
    i18n_domain = i18ndomain    
    manage_options = SubMenu.manage_options +\
                   ({'label':'Export',
                     'action':'exportSettings'},
                    )
    _properties = SubMenu._properties+\
                ({'id':'security_check', 'type': 'boolean','mode':'w',
                  'label':'Security check is enable'},)
    security = ClassSecurityInfo()  
    exportSettings = PageTemplateFile('zmi/exportSettings', globals())
    
    #security.declareProtected(ManagePortal, 'manage_export')
    def manage_export(self,export_path,REQUEST=None):
        """rakun applications export method"""
        message = StringIO()
        message.write("All menus are exported to %s\n"%export_path)
        for id in self.objectIds():             
            ob=self._getOb(id)
            f = path.join(export_path, '%s.%s' % (id, 'zexp'))
            ob._p_jar.exportFile(ob._p_oid, f)
            message.write("%s \n"%(f))
            
        if REQUEST:            
            return self.exportSettings(self,REQUEST,
                                       manage_tabs_message=message.getvalue())
        
    
InitializeClass(MenuTool)    