###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

from Products.PageTemplates.GlobalTranslationService import \
     getGlobalTranslationService, DummyTranslationService

service = None
translate = None

def get_i18n_language(context,domain):
    request = context.REQUEST
    language = request.get('TARGET_LANGUAGE',None)    
    if not language:
        ts = getGlobalTranslationService()
        language = ts.negotiate_language(context,domain)
        request.set('TARGET_LANGUAGE',language)
    return language   

def translate_wrapper(domain, msgid, mapping=None, context=None,
                      target_language=None, default=None,as_unicode=0):
    # wrapper for calling the translate() method with a fallback value
    if service == None:
        initialize()
    if not target_language and context:
        target_language = get_i18n_language(context, domain)
    if not default:
        default = msgid
    if as_unicode:
        service_translate = service.utranslate
    else:
        service_translate = service.translate
    try:
        res = service_translate(domain, msgid, mapping=mapping,
                                context=context,
                                target_language=target_language,
                                default=default)
    except TypeError:
        #Localizer does not take a default param
        res = service_translate(domain, msgid, mapping=mapping,
                                context=context,
                                target_language=target_language)

    if res is None or res is msgid:
        return default
    return res

def null_translate(domain, msgid, mapping=None, context=None,
                   target_language=None, default=None):
    return default

def initialize():
    # IMPORTANT: this module is unusable before this is called
    # this must be so because we want to make sure all products
    # (eg, whatever translation service we're supposed to use)
    # is already there and ready
    global service, translate
    service = getGlobalTranslationService()
    if service is DummyTranslationService:
        translate = null_translate
    elif hasattr(service, '_fallbacks'):
        # it accepts the "default" argument
        translate = service.translate
    else:
        translate = translate_wrapper

def initial_translate(domain, msgid, mapping=None, context=None,
                      target_language=None, default=None,as_unicode=0):
    initialize()
    return translate(domain, msgid, mapping, context, target_language, default,as_unicode)

translate = initial_translate

