###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

from Products.Formulator.Widget import Widget, TextWidget, render_element
from DateTime import DateTime

class FileWidget(TextWidget):
  
  def render_data(self, field, key, value, REQUEST):
    primary_value = REQUEST.get('primary_value','')
    href = "%s/getBlobObject?primary_value=%s"%(field.absolute_url(),
                                                primary_value)
    args = {"href"    : href,
            "target"  : "_blank",
            "contents": value,
            "class"   : "fileview" }    
    return render_element("a",**args)
  
  def render_helper(self, field, key, value, REQUEST):
    primary_value = REQUEST.get('primary_value','')
    src = "%s/filehelper_template?primary_value=%s"%(field.absolute_url(),
                                                     primary_value)
    args = {"class"             : "filehelperframe",
            "src"               : src,
            "border"            : 0,
            "frameborder"       : 0,
            "allowtransparency" : "true",
            "scrolling"         : "no",
            "marginheight"      : 0,
            "marginwidth"       : 0,
            "vspace"            : 0,
            "name"              : "filehelper_iframe",
            "id"                : "filehelper_iframe_%s" %field.getId(),
            "contents"          : "&nbsp;" }
    return render_element("iframe", **args)
                          

  def render(self, field, key, value, REQUEST):
    """Render file input field.
    """
    result = [] 
    primary_value = REQUEST.get('primary_value','')
   
    if primary_value:
      if value:
        result.append( self.render_data(field, key, value, REQUEST) )
        #result.append( render_element("br") )
        id = "filehelper_checkbox_%s" %field.getId()
        args = {"id":id, "type":"checkbox"}
        result.append( render_element("input", **args) )
        args = {"for":id, "contents":field.get_i18n_message("Delete"),
                "style":"padding-left:5px;"}
        result.append( render_element("label", **args) )
        result.append( render_element("br") )
      
    result.append( self.render_helper(field, key, value, REQUEST) )
    return "".join(result)
  
  def render_view(self, field, value):
    return "[File]"

FileWidgetInstance = FileWidget() 
  
  
class ImageWidget(FileWidget):
  
  def render_data(self, field, key, value, REQUEST):
    date = DateTime()
    primary_value = REQUEST.get('primary_value','')
    src = "%s/getBlobObject?primary_value=%s&extra=%s"%(field.absolute_url(),
                                                        primary_value,
                                                        str(date))
    args = {"src":src, "title":value, "alt":value, "class":"fileview" }
    if field.get_value("height"):
      args["height"] = "%spx" %field.get_value("height")
    if field.get_value("width"):
      args["width"] = "%spx" %field.get_value("width")
      
    return render_element("img",**args) 

  def render_view(self, field, value):
    return "[Image]"

ImageWidgetInstance = ImageWidget()   
      
      