###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

from Products.Rakun.Fields import BaseField
from Products.Rakun.Condition import Eq, Gt, Ge, Lt, Le, Ne, MatchGlobe
    
class Condition:
  
    simple_operators = ['=','<','<=','<>','>','>=','like']

      
    def __init__(self,field,operator,value):
        self.field = field
        self.value = value
        self.operator = operator
 
    def __and__(self,other):
        return MCondition(self,'and',other)
 
    def __or__(self,other):
        return MCondition(self,'or',other)
 
    def render(self, dbobject):
        if self.operator == '=':
            condition = Eq(self.field,self.value)
            
        elif self.operator == '>':
            condition = Gt(self.field,self.value)

        elif self.operator == '>=':
            condition = Ge(self.field,self.value)
            
        elif self.operator == '<':
            condition = Lt(self.field,self.value)
            
        elif self.operator == '<=':
            condition = Le(self.field,self.value)
            
        elif self.operator == '<>':
            condition = Ne(self.field,self.value)
        
        elif self.operator == 'like':
            condition = MatchGlobe(self.field,self.value)
        
        
        return condition.render(dbobject)
        
        
    def toRpc(self):
        """ rpcWrapper compatible dump """
        
        return {'__class__':'Condition',
                'args':[self.field, self.operator, self.value]}
           

class MCondition(Condition):
  
  
    def __init__(self,cond1,operator,cond2):
        self.cond1 = cond1
        self.cond2 = cond2
        self.operator = operator
    
    def render(self,dbobject):
        cond1 = self.cond1.render(dbobject)
        cond2 = self.cond2.render(dbobject)
        #operator = keywords.get(self.operator,self.operator)
        return '(%s %s %s)'%(cond1, self.operator, cond2)  
     
    def toRpc(self):
        """ rpcWrapper compatible dump """
        cond1 = self.cond1.toRpc()
        cond2 = self.cond2.toRpc()
        return {'__class__':'MCondition',
                'args':[cond1, self.operator, cond2]}
