###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

try:
  from psycopg import Binary
except ImportError:
  Binary = lambda blob: blob  
from Sql99 import Sql99, registerSqlClass

ROW_NUMBER_FUNCTION = """
CREATE OR REPLACE FUNCTION rowNumber(integer,varchar) RETURNS integer AS $$
DECLARE
    rec RECORD;
    counter integer := 1;    
    id integer := $1;
    query varchar := $2;
BEGIN
    FOR rec IN EXECUTE query LOOP
        IF rec.id = id THEN
            RETURN counter;
        ELSE
            counter := counter +1;
        END IF;
    END LOOP;  
    RETURN -1;
END;
$$ LANGUAGE plpgsql;"""


class Postgresql(Sql99):

  
    __implements__ = (Sql99.__implements__,)
    
    like_keyword = 'ilike'
  
    def getResultName(self, fieldid):
        return fieldid.lower()
  
    def getAutoIncrementValue(self,tableid,fieldid):
        query = "select nextval('%s_%s_seq') as nextval" % (tableid,fieldid)
        rec = self.execQuery(query)
        if rec: 
            return rec[0].nextval
        else:
            return 0
      
    def getLastIncrementValue(self,tableid,fieldid):
        query = "select currval('%s_%s_seq') as currval" % (tableid,fieldid)
        rec = self.execQuery(query)
        if rec:
            return rec[0].currval
        else:
            return 0
      
    def escapeBlob(self, blob):        
        return Binary(blob)
      
    def getRowNumber(self, entity, primary_value, fields, condition=None, orderby=[],
                     groupby=[]):
      query,real_names = self.renderSelectQuery(entity, fields, condition, orderby, groupby)
      query = "select rowNumber(%s,'%s')"%(primary_value, query)
      try:      
        result = self.execQuery(query)[0].rownumber      
      except:
        try:
          self.execQuery(ROW_NUMBER_FUNCTION)          
          result = self.execQuery(query)[0].rownumber       
        except:
          result = -1
      return result

registerSqlClass('postgresql',Postgresql)